@@ -22,21 +22,22 @@ class OutputType(StrEnum):
     TEXT = "text"
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class Goal:
     task: str
     instructions: list[str]
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class Role:
     persona: list[str]
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class Specification:
-    role: Role | None
-    goal: Goal | None
+    role: Role | None = None
+    goal: Goal | None = None
+    system_prompt: str | None = None
     rules: list[str] | None = field(default_factory=list)
     input_type: InputType = InputType.TEXT
     output_type: OutputType = OutputType.TEXT
@@ -45,13 +46,13 @@ class Specification:
     template_vars: dict | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class EngineEnvironment:
     engine_uri: EngineUri
     settings: EngineSettings | TransformerEngineSettings
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class AgentOperation:
     specification: Specification
     environment: EngineEnvironment
