@@ -5,26 +5,28 @@ from ..entities import (
     EngineUri,
     OrchestratorSettings,
     TransformerEngineSettings,
+    ToolFormat,
     ToolManagerSettings,
 )
 from ..event.manager import EventManager
 from ..memory.manager import MemoryManager
 from ..memory.partitioner.text import TextPartitioner
+from ..memory.permanent.pgsql.raw import PgsqlRawMemory
 from ..model.hubs.huggingface import HuggingfaceHub
 from ..model.manager import ModelManager
 from ..model.nlp.sentence import SentenceTransformerModel
 from ..tool.browser import BrowserToolSet, BrowserToolSettings
+from ..tool.code import CodeToolSet
+from ..tool.database import DatabaseToolSet, DatabaseToolSettings
 from ..tool.manager import ToolManager
 from ..tool.math import MathToolSet
 from ..tool.memory import MemoryToolSet
-from ..tool.code import CodeToolSet
-from ..memory.permanent.pgsql.raw import PgsqlRawMemory
 from contextlib import AsyncExitStack
-from logging import Logger
+from logging import Logger, DEBUG, INFO
 from os import access, R_OK
 from os.path import exists
 from tomllib import load
-from types import FunctionType
+from typing import Any, Callable
 from uuid import UUID, uuid4
 
 
@@ -66,7 +68,7 @@ class OrchestratorLoader:
         elif not access(path, R_OK):
             raise PermissionError(path)
 
-        _l("Loading agent from %s", path)
+        _l("Loading agent from %s", path, is_debug=False)
 
         with open(path, "rb") as file:
             config = load(file)
@@ -82,10 +84,9 @@ class OrchestratorLoader:
             ), "No uri defined in engine section of configuration"
 
             agent_config = config["agent"]
-            for setting in ["role"]:
-                assert (
-                    setting in agent_config
-                ), f"No {setting} defined in agent section of configuration"
+            assert not (
+                "user" in agent_config and "user_template" in agent_config
+            ), "user and user_template are mutually exclusive"
 
             assert (
                 "engine" in config
@@ -120,15 +121,10 @@ class OrchestratorLoader:
                 f"Unknown type {config['agent']['type']} in agent section "
                 + "of configuration"
             )
-            assert (
-                "role" in agent_config
-            ), "No role defined in agent section of configuration"
 
             call_options = config["run"] if "run" in config else None
             if call_options and "chat" in call_options:
-                call_options["chat_template_settings"] = call_options.pop(
-                    "chat"
-                )
+                call_options["chat_settings"] = call_options.pop("chat")
             template_vars = (
                 config["template"] if "template" in config else None
             )
@@ -223,22 +219,37 @@ class OrchestratorLoader:
                 log_events=True,
             )
 
-            tool_config = config.get("tool", {}).get("browser", {}).get("open")
-            if not tool_config and "browser" in config.get("tool", {}):
-                tool_config = config["tool"]["browser"]
+            tool_section = config.get("tool", {})
+            browser_config = tool_section.get("browser", {}).get("open")
+            if not browser_config and "browser" in tool_section:
+                browser_config = tool_section["browser"]
             browser_settings = None
-            if tool_config:
-                if "debug_source" in tool_config and isinstance(
-                    tool_config["debug_source"], str
+            if browser_config:
+                if "debug_source" in browser_config and isinstance(
+                    browser_config["debug_source"], str
                 ):
-                    tool_config["debug_source"] = open(
-                        tool_config["debug_source"]
+                    browser_config["debug_source"] = open(
+                        browser_config["debug_source"]
                     )
-                browser_settings = BrowserToolSettings(**tool_config)
+                browser_settings = BrowserToolSettings(**browser_config)
+
+            database_settings = None
+            database_config = tool_section.get("database")
+            if database_config:
+                database_settings = DatabaseToolSettings(**database_config)
+
+            tool_format = None
+            tool_format_str = tool_section.get("format")
+            if tool_format_str:
+                tool_format = ToolFormat(tool_format_str)
+
+            _l("Loaded agent from %s", path, is_debug=False)
 
             return await self.from_settings(
                 settings,
                 browser_settings=browser_settings,
+                database_settings=database_settings,
+                tool_format=tool_format,
             )
 
     async def from_settings(
@@ -246,9 +257,13 @@ class OrchestratorLoader:
         settings: OrchestratorSettings,
         *,
         browser_settings: BrowserToolSettings | None = None,
+        database_settings: DatabaseToolSettings | None = None,
+        tool_format: ToolFormat | None = None,
     ) -> Orchestrator:
         _l = self._log_wrapper(self._logger)
 
+        _l("Loading agent from settings", is_debug=False)
+
         sentence_model_engine_settings = (
             TransformerEngineSettings(**settings.sentence_model_engine_config)
             if settings.sentence_model_engine_config
@@ -337,11 +352,17 @@ class OrchestratorLoader:
             MathToolSet(namespace="math"),
             MemoryToolSet(memory, namespace="memory"),
         ]
+        if database_settings:
+            available_toolsets.append(
+                DatabaseToolSet(
+                    settings=database_settings, namespace="database"
+                )
+            )
 
         tool = ToolManager.create_instance(
             available_toolsets=available_toolsets,
             enable_tools=settings.tools,
-            settings=ToolManagerSettings(),
+            settings=ToolManagerSettings(tool_format=tool_format),
         )
         tool = await self._stack.enter_async_context(tool)
 
@@ -391,18 +412,31 @@ class OrchestratorLoader:
                 id=settings.agent_id,
                 name=settings.agent_config.get("name"),
                 role=(
-                    settings.agent_config["role"]
-                    if "role" in settings.agent_config
-                    else None
+                    None
+                    if "system" in settings.agent_config
+                    else settings.agent_config.get("role")
+                ),
+                task=(
+                    None
+                    if "system" in settings.agent_config
+                    else settings.agent_config.get("task")
+                ),
+                instructions=(
+                    None
+                    if "system" in settings.agent_config
+                    else settings.agent_config.get("instructions")
                 ),
-                task=settings.agent_config.get("task"),
-                instructions=settings.agent_config.get("instructions"),
                 rules=settings.agent_config.get("rules"),
+                system=settings.agent_config.get("system"),
+                user=settings.agent_config.get("user"),
+                user_template=settings.agent_config.get("user_template"),
                 settings=engine_settings,
                 call_options=settings.call_options,
                 template_vars=settings.template_vars,
             )
 
+        _l("Loaded agent from settings", is_debug=False)
+
         return agent
 
     @staticmethod
@@ -421,15 +455,13 @@ class OrchestratorLoader:
         template_vars: dict | None,
     ) -> JsonOrchestrator:
         assert "json" in config, "No json section in configuration"
-        assert (
-            "instructions" in agent_config
-        ), "No instructions defined in agent section of configuration"
-        assert (
-            "task" in agent_config
-        ), "No task defined in agent section of configuration"
-        assert (
-            "role" in agent_config
-        ), "No role defined in agent section of configuration"
+        if "system" not in agent_config:
+            assert (
+                "instructions" in agent_config
+            ), "No instructions defined in agent section of configuration"
+            assert (
+                "task" in agent_config
+            ), "No task defined in agent section of configuration"
 
         properties: list[Property] = []
         for property_name in config.get("json", []):
@@ -454,10 +486,21 @@ class OrchestratorLoader:
             properties,
             id=agent_id,
             name=agent_config["name"] if "name" in agent_config else None,
-            role=agent_config["role"],
-            task=agent_config["task"],
-            instructions=agent_config["instructions"],
-            rules=agent_config["rules"] if "rules" in agent_config else None,
+            role=(
+                None if "system" in agent_config else agent_config.get("role")
+            ),
+            task=(
+                None if "system" in agent_config else agent_config.get("task")
+            ),
+            instructions=(
+                None
+                if "system" in agent_config
+                else agent_config.get("instructions")
+            ),
+            rules=agent_config.get("rules"),
+            system=agent_config.get("system"),
+            user=agent_config.get("user"),
+            user_template=agent_config.get("user_template"),
             settings=engine_settings,
             call_options=call_options,
             template_vars=template_vars,
@@ -465,14 +508,20 @@ class OrchestratorLoader:
         return agent
 
     @staticmethod
-    def _log_wrapper(logger: Logger) -> FunctionType:
-        return lambda message, *args, inner_type=None, **kwargs: logger.debug(
-            (
+    def _log_wrapper(logger: Logger) -> Callable[..., Any]:
+        def wrapper(
+            message: str,
+            *args: Any,
+            inner_type: str | None = None,
+            **kwargs: Any,
+        ) -> Any:
+            is_debug = kwargs.pop("is_debug", True)
+            level = DEBUG if is_debug else INFO
+            prefix = (
                 f"<{inner_type} @ OrchestratorLoader> "
                 if inner_type
                 else "<OrchestratorLoader> "
             )
-            + message,
-            *args,
-            **kwargs,
-        )
+            return logger.log(level, prefix + message, *args, **kwargs)
+
+        return wrapper
