@@ -10,6 +10,7 @@ from ..cli.commands.agent import (
     agent_message_search,
     agent_run,
     agent_serve,
+    agent_proxy,
     agent_init,
 )
 from ..cli.commands.cache import cache_delete, cache_download, cache_list
@@ -39,8 +40,11 @@ from ..entities import (
     TimestepSpacing,
     TextGenerationLoaderClass,
     Backend,
+    ReasoningTag,
+    GenerationCacheStrategy,
     User,
     WeightType,
+    ToolFormat,
 )
 from ..memory.permanent import VectorFunction
 from ..model.hubs.huggingface import HuggingfaceHub
@@ -78,11 +82,14 @@ from transformers.utils import (
 from typing import get_args, Optional, get_origin, get_args as get_type_args
 from dataclasses import fields
 from ..tool.browser import BrowserToolSettings
+from ..tool.database import DatabaseToolSettings
 from uuid import uuid4
 from warnings import filterwarnings
 
 
 class CLI:
+    """Command line interface entry point."""
+
     _REFRESH_RATE = 4
 
     def __init__(self, logger: Logger):
@@ -252,6 +259,11 @@ class CLI:
             )
             + " automatically)",
         )
+        global_parser.add_argument(
+            "--tty",
+            default="/dev/tty",
+            help="TTY stream to use for interactive prompts",
+        )
         global_parser.add_argument(
             "--record",
             action="store_true",
@@ -339,6 +351,31 @@ class CLI:
             help="Number of tokens per window when partitioning",
         )
 
+        # Model options shared by commands: cache download, model install
+        model_install_parser = ArgumentParser(add_help=False)
+        model_install_parser.add_argument(
+            "model",
+            type=str,
+            help="Model to download",
+        )
+        model_install_parser.add_argument(
+            "--workers",
+            default=8,
+            type=int,
+            help="How many download workers to use",
+        )
+        model_install_parser.add_argument(
+            "--local-dir",
+            type=str,
+            help="Local directory to download the model to",
+        )
+        model_install_parser.add_argument(
+            "--local-dir-symlinks",
+            action="store_true",
+            default=None,
+            help="Use symlinks when downloading to local dir",
+        )
+
         # Model options shared by commands: memory embeddings, model
         model_options_parser = ArgumentParser(add_help=False)
         model_options_parser.add_argument(
@@ -442,6 +479,11 @@ class CLI:
                 "(defaults to 256)"
             ),
         )
+        model_inference_display_parser.add_argument(
+            "--skip-display-reasoning-time",
+            action="store_true",
+            help="Don't display total reasoning time",
+        )
         model_inference_display_parser.add_argument(
             "--display-tokens",
             type=int,
@@ -465,6 +507,21 @@ class CLI:
             help="How many tool events to show on tool call panel",
         )
 
+        display_answer_height_group = (
+            model_inference_display_parser.add_mutually_exclusive_group()
+        )
+        display_answer_height_group.add_argument(
+            "--display-answer-height-expand",
+            action="store_true",
+            help="Expand answer section to full height",
+        )
+        display_answer_height_group.add_argument(
+            "--display-answer-height",
+            type=int,
+            default=12,
+            help="Height of the answer section (defaults to 12)",
+        )
+
         # Agent command
         agent_parser = command_parsers.add_parser(
             name="agent",
@@ -529,18 +586,40 @@ class CLI:
             prefix="browser",
             settings_cls=BrowserToolSettings,
         )
-
-        agent_run_parser = agent_command_parsers.add_parser(
-            name="run",
-            description="Run an AI agent",
-            parents=[global_parser, model_inference_display_parser],
+        CLI._add_tool_settings_arguments(
+            agent_message_search_parser,
+            prefix="database",
+            settings_cls=DatabaseToolSettings,
         )
-        agent_run_parser.add_argument(
+
+        agent_common_parser = ArgumentParser(add_help=False)
+        agent_common_parser.add_argument(
             "specifications_file",
             type=str,
             nargs="?",
             help="File that holds the agent specifications",
         )
+        agent_common_parser.add_argument(
+            "--id", type=str, help="Use given ID as the agent ID"
+        )
+        agent_common_parser.add_argument(
+            "--participant",
+            default=uuid4(),
+            help=(
+                "If specified, this is the participant ID interacting with "
+                "the agent"
+            ),
+        )
+
+        agent_run_parser = agent_command_parsers.add_parser(
+            name="run",
+            description="Run an AI agent",
+            parents=[
+                global_parser,
+                model_inference_display_parser,
+                agent_common_parser,
+            ],
+        )
         agent_run_parser.add_argument(
             "--conversation",
             action="store_true",
@@ -556,9 +635,6 @@ class CLI:
                 "(only with --conversation)"
             ),
         )
-        agent_run_parser.add_argument(
-            "--id", type=str, help="Use given ID as the agent ID"
-        )
         agent_session_group = agent_run_parser.add_mutually_exclusive_group()
         agent_session_group.add_argument(
             "--no-session",
@@ -585,14 +661,6 @@ class CLI:
             type=int,
             help="If specified, load up to these many recent messages",
         )
-        agent_run_parser.add_argument(
-            "--participant",
-            default=uuid4(),
-            help=(
-                "If specified, this is the participant ID interacting with "
-                "the agent"
-            ),
-        )
         agent_run_parser.add_argument(
             "--stats",
             action="store_true",
@@ -606,19 +674,23 @@ class CLI:
             default=False,
             help="Don't use an async generator (token streaming)",
         )
-        agent_run_parser.add_argument(
-            "--tty",
-            default="/dev/tty",
-            help=(
-                "TTY stream (only applicable if combining --conversation "
-                "with input piping)"
-            ),
-        )
         agent_run_parser.add_argument(
             "--tools-confirm",
             action="store_true",
             help="Confirm tool calls before execution",
         )
+        agent_run_parser.add_argument(
+            "--tool-format",
+            type=str,
+            choices=[t.value for t in ToolFormat],
+            help="Tool format",
+        )
+        agent_run_parser.add_argument(
+            "--reasoning-tag",
+            type=str,
+            choices=[t.value for t in ReasoningTag],
+            help="Reasoning tag style",
+        )
 
         CLI._add_agent_settings_arguments(agent_run_parser)
         CLI._add_tool_settings_arguments(
@@ -626,55 +698,47 @@ class CLI:
             prefix="browser",
             settings_cls=BrowserToolSettings,
         )
+        CLI._add_tool_settings_arguments(
+            agent_run_parser,
+            prefix="database",
+            settings_cls=DatabaseToolSettings,
+        )
 
         agent_serve_parser = agent_command_parsers.add_parser(
             name="serve",
             description="Serve an AI agent as an API endpoint",
-            parents=[global_parser],
+            parents=[global_parser, agent_common_parser],
         )
-        agent_serve_parser.add_argument(
-            "specifications_file",
-            type=str,
-            nargs="?",
-            help="File that holds the agent specifications",
-        )
-        agent_serve_parser.add_argument(
-            "--host",
-            default="127.0.0.1",
-            type=str,
-            help="Host (defaults to 127.0.0.1)",
-        )
-        agent_serve_parser.add_argument(
-            "--port",
-            default=9001,
-            type=int,
-            help="Port (defaults to 9001, HAL 9000+1)",
-        )
-        agent_serve_parser.add_argument(
-            "--prefix-mcp",
-            default="/mcp",
-            type=str,
-            help="URL prefix for MCP endpoints (defaults to /mcp)",
-        )
-        agent_serve_parser.add_argument(
-            "--prefix-openai",
-            default="/v1",
-            type=str,
-            help="URL prefix fir OpenAI endpoints (defaults to /v1)",
-        )
-        agent_serve_parser.add_argument(
-            "--reload",
-            action="store_true",
-            default=False,
-            help="Hot reload on code changes",
-        )
-
+        CLI._add_agent_server_arguments(agent_serve_parser)
         CLI._add_agent_settings_arguments(agent_serve_parser)
         CLI._add_tool_settings_arguments(
             agent_serve_parser,
             prefix="browser",
             settings_cls=BrowserToolSettings,
         )
+        CLI._add_tool_settings_arguments(
+            agent_serve_parser,
+            prefix="database",
+            settings_cls=DatabaseToolSettings,
+        )
+
+        agent_proxy_parser = agent_command_parsers.add_parser(
+            name="proxy",
+            description="Serve a proxy agent as an API endpoint",
+            parents=[global_parser, agent_common_parser],
+        )
+        CLI._add_agent_server_arguments(agent_proxy_parser)
+        CLI._add_agent_settings_arguments(agent_proxy_parser)
+        CLI._add_tool_settings_arguments(
+            agent_proxy_parser,
+            prefix="browser",
+            settings_cls=BrowserToolSettings,
+        )
+        CLI._add_tool_settings_arguments(
+            agent_proxy_parser,
+            prefix="database",
+            settings_cls=DatabaseToolSettings,
+        )
 
         agent_init_parser = agent_command_parsers.add_parser(
             name="init",
@@ -718,17 +782,10 @@ class CLI:
             action="append",
             help="Revision to delete",
         )
-        cache_download_parser = cache_command_parsers.add_parser(
+        cache_command_parsers.add_parser(
             name="download",
             description="Download model data to cache",
-            parents=[global_parser],
-        )
-        cache_download_parser.add_argument(
-            "--model",
-            "-m",
-            type=str,
-            required=True,
-            help="Model to load",
+            parents=[global_parser, model_install_parser],
         )
         cache_list_parser = cache_command_parsers.add_parser(
             name="list",
@@ -954,7 +1011,7 @@ class CLI:
         model_command_parsers.add_parser(
             name="install",
             description="Install a model",
-            parents=[global_parser, model_options_parser],
+            parents=[global_parser, model_install_parser],
         )
         model_run_parser = model_command_parsers.add_parser(
             name="run",
@@ -975,6 +1032,12 @@ class CLI:
                 f"(defaults to best available: {default_attention})"
             ),
         )
+        model_run_parser.add_argument(
+            "--output-hidden-states",
+            action="store_true",
+            default=False,
+            help="Return hidden states for each layer",
+        )
         model_run_parser.add_argument(
             "--path",
             type=str,
@@ -1312,6 +1375,19 @@ class CLI:
             default=1,
             help="Number of beams for beam search. 1 means no beam search",
         )
+        model_run_parser.add_argument(
+            "--text-disable-cache",
+            dest="use_cache",
+            action="store_false",
+            help="If specified, disable generation cache",
+        )
+        model_run_parser.add_argument(
+            "--text-cache-strategy",
+            type=str,
+            choices=[c.value for c in GenerationCacheStrategy],
+            dest="cache_strategy",
+            help="Cache implementation to use for generation",
+        )
         model_run_parser.add_argument(
             "--text-from-lang",
             type=str,
@@ -1328,6 +1404,36 @@ class CLI:
             action="store_true",
             help="If specified, assume model response starts with reasoning",
         )
+        model_run_parser.add_argument(
+            "--chat-disable-thinking",
+            dest="chat_disable_thinking",
+            action="store_true",
+            default=False,
+            help="Disable thinking tokens in chat template",
+        )
+        model_run_parser.add_argument(
+            "--no-reasoning",
+            action="store_true",
+            default=False,
+            help="Disable reasoning parser",
+        )
+        model_run_parser.add_argument(
+            "--reasoning-tag",
+            type=str,
+            choices=[t.value for t in ReasoningTag],
+            help="Reasoning tag style",
+        )
+        model_run_parser.add_argument(
+            "--reasoning-max-new-tokens",
+            type=int,
+            help="Maximum number of reasoning tokens",
+        )
+        model_run_parser.add_argument(
+            "--reasoning-stop-on-max-new-tokens",
+            action="store_true",
+            default=False,
+            help="Stop reasoning when maximum tokens are produced",
+        )
         model_run_parser.add_argument(
             "--stop_on_keyword",
             type=str,
@@ -1508,9 +1614,22 @@ class CLI:
         return parser
 
     @staticmethod
-    def _extract_chat_template_settings(
+    def _get_translator(
+        app_name: str, locales_path: str, locale: str
+    ) -> object:
+        """Return translation object for ``locale`` or ``gettext`` fallback."""
+        try:
+            return translation(
+                app_name, localedir=locales_path, languages=[locale]
+            )
+        except FileNotFoundError:
+            return gettext
+
+    @staticmethod
+    def _extract_chat_settings(
         argv: list[str],
     ) -> tuple[list[str], dict[str, bool]]:
+        """Return ``argv`` without chat options and extracted flags."""
         options: dict[str, bool] = {}
         new_argv: list[str] = []
         for arg in argv:
@@ -1521,6 +1640,41 @@ class CLI:
                 new_argv.append(arg)
         return new_argv, options
 
+    @staticmethod
+    def _add_agent_server_arguments(parser: ArgumentParser) -> ArgumentParser:
+        """Add shared server options for agent commands."""
+        parser.add_argument(
+            "--host",
+            default="127.0.0.1",
+            type=str,
+            help="Host (defaults to 127.0.0.1)",
+        )
+        parser.add_argument(
+            "--port",
+            default=9001,
+            type=int,
+            help="Port (defaults to 9001, HAL 9000+1)",
+        )
+        parser.add_argument(
+            "--prefix-mcp",
+            default="/mcp",
+            type=str,
+            help="URL prefix for MCP endpoints (defaults to /mcp)",
+        )
+        parser.add_argument(
+            "--prefix-openai",
+            default="/v1",
+            type=str,
+            help="URL prefix fir OpenAI endpoints (defaults to /v1)",
+        )
+        parser.add_argument(
+            "--reload",
+            action="store_true",
+            default=False,
+            help="Hot reload on code changes",
+        )
+        return parser
+
     @staticmethod
     def _add_agent_settings_arguments(
         parser: ArgumentParser,
@@ -1533,6 +1687,10 @@ class CLI:
         group.add_argument(
             "--instructions", type=str, help="Agent instructions"
         )
+        group.add_argument("--user", type=str, help="User message template")
+        group.add_argument(
+            "--user-template", type=str, help="User message template file"
+        )
         group.add_argument(
             "--memory-recent",
             dest="memory_recent",
@@ -1589,6 +1747,20 @@ class CLI:
             default=False,
             help="Skip special tokens on output",
         )
+        group.add_argument(
+            "--run-disable-cache",
+            dest="run_use_cache",
+            action="store_false",
+            default=None,
+            help="Disable generation cache",
+        )
+        group.add_argument(
+            "--run-cache-strategy",
+            type=str,
+            choices=[c.value for c in GenerationCacheStrategy],
+            dest="run_cache_strategy",
+            help="Cache implementation to use for generation",
+        )
         group.add_argument(
             "--run-temperature",
             default=0.7,
@@ -1615,12 +1787,19 @@ class CLI:
         group.add_argument(
             "--tool", type=str, action="append", help="Enable tool"
         )
+        group.add_argument(
+            "--tools",
+            type=str,
+            action="append",
+            help="Enable tools matching namespace",
+        )
         return group
 
     @staticmethod
     def _add_tool_settings_arguments(
         parser: ArgumentParser, *, prefix: str, settings_cls: type
     ) -> ArgumentParser:
+        """Add dataclass based tool options to ``parser``."""
         group = parser.add_argument_group(f"{prefix} tool settings")
 
         for field in fields(settings_cls):
@@ -1653,12 +1832,13 @@ class CLI:
 
     @staticmethod
     def _needs_hf_token(args: Namespace) -> bool:
+        """Return ``True`` if the command needs hub authentication."""
         command = args.command
         if command == "model" and (args.model_command or "display") == "run":
             engine_uri = ModelManager.parse_uri(args.model)
             return engine_uri.is_local
         if command == "agent" and (
-            (args.agent_command or "run") in {"run", "serve"}
+            (args.agent_command or "run") in {"run", "serve", "proxy"}
         ):
             engine = getattr(args, "engine_uri", None)
             if engine:
@@ -1667,7 +1847,7 @@ class CLI:
         return True
 
     async def __call__(self) -> None:
-        argv, chat_opts = self._extract_chat_template_settings(sys.argv[1:])
+        argv, chat_opts = self._extract_chat_settings(sys.argv[1:])
         args = self._parser.parse_args(argv)
 
         if args.parallel and not args.quiet:
@@ -1699,13 +1879,7 @@ class CLI:
         for key, value in chat_opts.items():
             setattr(args, f"run_chat_{key}", value)
 
-        current_locale, _ = getlocale()
-        try:
-            translator = translation(
-                self._name, localedir=args.locales, languages=[args.locale]
-            )
-        except FileNotFoundError:
-            translator = gettext
+        translator = CLI._get_translator(self._name, args.locales, args.locale)
 
         assert self._logger is not None and isinstance(self._logger, Logger)
         theme = FancyTheme(translator.gettext, translator.ngettext)
@@ -1725,7 +1899,7 @@ class CLI:
                 prompt_kwargs = {}
                 if has_input(console):
                     try:
-                        prompt_kwargs["stream"] = open("/dev/tty")
+                        prompt_kwargs["stream"] = open(args.tty)
                     except OSError:
                         pass
                 access_token = Prompt.ask(
@@ -1750,8 +1924,15 @@ class CLI:
                 pass
 
     def _help(
-        self, console: Console, parser: ArgumentParser, path: list[str] = []
+        self,
+        console: Console,
+        parser: ArgumentParser,
+        path: list[str] | None = None,
     ) -> None:
+        """Recursively output help information for ``parser``."""
+        if path is None:
+            path = []
+
         prog = parser.prog
         is_root_command = not path
         console.print(
@@ -1871,6 +2052,14 @@ class CLI:
                             self._name,
                             str(self._version),
                         )
+                    case "proxy":
+                        await agent_proxy(
+                            args,
+                            hub,
+                            self._logger,
+                            self._name,
+                            str(self._version),
+                        )
                     case "init":
                         await agent_init(args, console, theme)
             case "cache":
@@ -1935,6 +2124,7 @@ class CLI:
 
 
 def main() -> None:
+    """Entry point for the ``avalan`` CLI."""
     basicConfig(
         level=INFO,
         format="%(message)s",
