@@ -3,20 +3,27 @@ from ...agent.orchestrator import Orchestrator
 from ...agent.orchestrator.response.orchestrator_response import (
     OrchestratorResponse,
 )
-from ...cli import get_input, has_input, confirm_tool_call
+from ...cli import confirm_tool_call, get_input, has_input
 from ...cli.commands.model import token_generation
-from ...entities import OrchestratorSettings, ToolCall
+from ...entities import (
+    Backend,
+    GenerationCacheStrategy,
+    OrchestratorSettings,
+    ToolCall,
+    ToolFormat,
+)
 from ...event import EventStats
 from ...model.hubs.huggingface import HuggingfaceHub
 from ...model.nlp.text.vendor import TextGenerationVendorModel
 from ...server import agents_server
 from ...tool.browser import BrowserToolSettings
+from ...tool.database import DatabaseToolSettings
 from argparse import Namespace
 from contextlib import AsyncExitStack
 from dataclasses import fields
 from jinja2 import Environment, FileSystemLoader
 from logging import Logger
-from os.path import dirname, join, getmtime
+from os.path import dirname, getmtime, join
 from rich.console import Console
 from rich.prompt import Confirm, Prompt
 from rich.syntax import Syntax
@@ -33,6 +40,8 @@ def get_orchestrator_settings(
     role: str | None = None,
     task: str | None = None,
     instructions: str | None = None,
+    user: str | None = None,
+    user_template: str | None = None,
     engine_uri: str | None = None,
     memory_recent: bool | None = None,
     memory_permanent_message: str | None = None,
@@ -42,8 +51,14 @@ def get_orchestrator_settings(
     tools: list[str] | None = None,
     top_k: int | None = None,
     top_p: float | None = None,
+    use_cache: bool | None = None,
+    cache_strategy: GenerationCacheStrategy | None = None,
 ) -> OrchestratorSettings:
     """Create ``OrchestratorSettings`` from CLI arguments."""
+    assert not (
+        (user or getattr(args, "user", None))
+        and (user_template or getattr(args, "user_template", None))
+    )
     memory_recent = (
         memory_recent
         if memory_recent is not None
@@ -60,11 +75,23 @@ def get_orchestrator_settings(
         else args.run_max_new_tokens
     )
 
-    chat_template_settings = {
+    chat_settings = {
         k[len("run_chat_") :]: v
         for k, v in vars(args).items()
         if k.startswith("run_chat_") and v is not None
     }
+    call_options = {
+        "max_new_tokens": call_tokens,
+        "skip_special_tokens": args.run_skip_special_tokens,
+        "temperature": temperature,
+        "top_k": top_k,
+        "top_p": top_p,
+        **({"chat_settings": chat_settings} if chat_settings else {}),
+    }
+    if use_cache is not None:
+        call_options["use_cache"] = use_cache
+    if cache_strategy is not None:
+        call_options["cache_strategy"] = cache_strategy
 
     return OrchestratorSettings(
         agent_id=agent_id,
@@ -80,23 +107,22 @@ def get_orchestrator_settings(
                     if instructions is not None
                     else args.instructions
                 ),
+                "user": (
+                    user if user is not None else getattr(args, "user", None)
+                ),
+                "user_template": (
+                    user_template
+                    if user_template is not None
+                    else getattr(args, "user_template", None)
+                ),
             }.items()
             if v is not None
         },
         uri=engine_uri,
-        engine_config=None,
-        call_options={
-            "max_new_tokens": call_tokens,
-            "skip_special_tokens": args.run_skip_special_tokens,
-            "temperature": temperature,
-            "top_k": top_k,
-            "top_p": top_p,
-            **(
-                {"chat_template_settings": chat_template_settings}
-                if chat_template_settings
-                else {}
-            ),
+        engine_config={
+            "backend": getattr(args, "backend", Backend.TRANSFORMERS.value)
         },
+        call_options=call_options,
         template_vars=None,
         memory_permanent_message=(
             memory_permanent_message
@@ -126,7 +152,11 @@ def get_orchestrator_settings(
         sentence_model_overlap_size=args.memory_engine_overlap,
         sentence_model_window_size=args.memory_engine_window,
         json_config=None,
-        tools=tools if tools is not None else args.tool or [],
+        tools=(
+            tools
+            if tools is not None
+            else (args.tool or []) + (getattr(args, "tools", None) or [])
+        ),
         log_events=True,
     )
 
@@ -206,11 +236,14 @@ async def agent_message_search(
 
     assert agent_id and participant_id and session_id
 
+    tty_path = getattr(args, "tty", "/dev/tty") or "/dev/tty"
+
     input_string = get_input(
         console,
         _i["user_input"] + " ",
         echo_stdin=not args.no_repl,
         is_quiet=args.quiet,
+        tty_path=tty_path,
     )
     if not input_string:
         return
@@ -254,14 +287,19 @@ async def agent_message_search(
                     args,
                     agent_id=agent_id,
                     memory_recent=memory_recent,
-                    tools=args.tool,
+                    tools=(args.tool or [])
+                    + (getattr(args, "tools", None) or []),
                 )
                 browser_settings = get_tool_settings(
                     args, prefix="browser", settings_cls=BrowserToolSettings
                 )
+                database_settings = get_tool_settings(
+                    args, prefix="database", settings_cls=DatabaseToolSettings
+                )
                 orchestrator = await loader.from_settings(
                     settings,
                     browser_settings=browser_settings,
+                    database_settings=database_settings,
                 )
             orchestrator = await stack.enter_async_context(orchestrator)
 
@@ -337,9 +375,10 @@ async def agent_run(
     load_recent_messages_limit = args.load_recent_messages_limit
 
     event_stats = EventStats()
+    tty_path = getattr(args, "tty", "/dev/tty") or "/dev/tty"
 
     def _confirm_call(call: ToolCall) -> str:
-        return confirm_tool_call(console, call)
+        return confirm_tool_call(console, call, tty_path=tty_path)
 
     async def _event_listener(event):
         nonlocal event_stats
@@ -370,8 +409,8 @@ async def agent_run(
             )
         else:
             assert (
-                args.engine_uri and args.role
-            ), "--engine-uri and --role required when no specifications file"
+                args.engine_uri
+            ), "--engine-uri required when no specifications file"
             assert not args.specifications_file or not args.engine_uri
             memory_recent = (
                 args.memory_recent
@@ -382,15 +421,29 @@ async def agent_run(
                 args,
                 agent_id=agent_id or uuid4(),
                 memory_recent=memory_recent,
-                tools=args.tool,
+                tools=(args.tool or []) + (getattr(args, "tools", None) or []),
+                max_new_tokens=getattr(args, "run_max_new_tokens", None),
+                temperature=getattr(args, "run_temperature", None),
+                top_k=getattr(args, "run_top_k", None),
+                top_p=getattr(args, "run_top_p", None),
+                use_cache=getattr(args, "run_use_cache", None),
+                cache_strategy=getattr(args, "run_cache_strategy", None),
             )
             logger.debug("Loading agent from inline settings")
             browser_settings = get_tool_settings(
                 args, prefix="browser", settings_cls=BrowserToolSettings
             )
+            database_settings = get_tool_settings(
+                args, prefix="database", settings_cls=DatabaseToolSettings
+            )
+            tool_format = (
+                ToolFormat(args.tool_format) if args.tool_format else None
+            )
             orchestrator = await loader.from_settings(
                 settings,
                 browser_settings=browser_settings,
+                database_settings=database_settings,
+                tool_format=tool_format,
             )
         orchestrator.event_manager.add_listener(_event_listener)
 
@@ -498,7 +551,7 @@ async def agent_run(
                 echo_stdin=not args.no_repl,
                 force_prompt=in_conversation,
                 is_quiet=args.quiet,
-                tty_path=args.tty,
+                tty_path=tty_path,
             )
             if not input_string:
                 logger.debug("Finishing session with orchestrator")
@@ -554,6 +607,8 @@ async def agent_serve(
 ) -> None:
     assert args.host and args.port
     specs_path = args.specifications_file
+    agent_id = getattr(args, "id", None)
+    participant_id = args.participant
     engine_uri = getattr(args, "engine_uri", None)
     assert not (
         specs_path and engine_uri
@@ -562,61 +617,70 @@ async def agent_serve(
         specs_path or engine_uri
     ), "specifications file or --engine-uri must be specified"
 
-    async with AsyncExitStack() as stack:
-        loader = OrchestratorLoader(
-            hub=hub,
-            logger=logger,
-            participant_id=uuid4(),
-            stack=stack,
-        )
-        if specs_path:
-            logger.debug("Loading agent from %s", specs_path)
+    settings: OrchestratorSettings | None = None
+    browser_settings: BrowserToolSettings | None = None
+    database_settings: DatabaseToolSettings | None = None
 
-            orchestrator = await loader.from_file(
-                specs_path,
-                agent_id=uuid4(),
-            )
-        else:
-            assert args.role
-            memory_recent = (
-                args.memory_recent if args.memory_recent is not None else True
-            )
-            settings = get_orchestrator_settings(
-                args,
-                agent_id=uuid4(),
-                memory_recent=memory_recent,
-                tools=args.tool,
-            )
-            logger.debug("Loading agent from inline settings")
-            browser_settings = get_tool_settings(
-                args, prefix="browser", settings_cls=BrowserToolSettings
-            )
-            orchestrator = await loader.from_settings(
-                settings,
-                browser_settings=browser_settings,
-            )
-        orchestrator = await stack.enter_async_context(orchestrator)
+    if not specs_path:
+        memory_recent = (
+            args.memory_recent if args.memory_recent is not None else True
+        )
+        settings = get_orchestrator_settings(
+            args,
+            agent_id=agent_id or uuid4(),
+            memory_recent=memory_recent,
+            tools=(args.tool or []) + (getattr(args, "tools", None) or []),
+        )
+        browser_settings = get_tool_settings(
+            args, prefix="browser", settings_cls=BrowserToolSettings
+        )
+        database_settings = get_tool_settings(
+            args, prefix="database", settings_cls=DatabaseToolSettings
+        )
 
-        logger.debug(
-            "Agent loaded from"
-            f" {specs_path if specs_path else 'inline settings'}"
-        )
-        server = agents_server(
-            name=name,
-            version=version,
-            prefix_openai=args.prefix_openai,
-            prefix_mcp=args.prefix_mcp,
-            orchestrators=[orchestrator],
-            host=args.host,
-            port=args.port,
-            reload=args.reload,
-            logger=logger,
-        )
-        await server.serve()
+    server = agents_server(
+        hub=hub,
+        name=name,
+        version=version,
+        prefix_openai=args.prefix_openai,
+        prefix_mcp=args.prefix_mcp,
+        specs_path=specs_path,
+        settings=settings,
+        browser_settings=browser_settings,
+        database_settings=database_settings,
+        host=args.host,
+        port=args.port,
+        reload=args.reload,
+        logger=logger,
+        agent_id=agent_id,
+        participant_id=participant_id,
+    )
+    await server.serve()
+
+
+async def agent_proxy(
+    args: Namespace,
+    hub: HuggingfaceHub,
+    logger: Logger,
+    name: str,
+    version: str,
+) -> None:
+    args.name = getattr(args, "name", "Proxy") or "Proxy"
+    args.memory_recent = getattr(args, "memory_recent", True) or True
+    args.memory_permanent_message = (
+        getattr(args, "memory_permanent_message", None)
+        or "postgresql://avalan:password@localhost:5432/avalan"
+    )
+    args.specifications_file = None
+
+    assert getattr(args, "engine_uri", None), "--engine-uri is required"
+
+    await agent_serve(args, hub, logger, name, version)
 
 
 async def agent_init(args: Namespace, console: Console, theme: Theme) -> None:
     _ = theme._
+    tty_path = getattr(args, "tty", "/dev/tty") or "/dev/tty"
 
     name = args.name or Prompt.ask(_("Agent name"))
     role = args.role or get_input(
@@ -624,21 +688,22 @@ async def agent_init(args: Namespace, console: Console, theme: Theme) -> None:
         _("Agent role") + " ",
         echo_stdin=not args.no_repl,
         is_quiet=args.quiet,
+        tty_path=tty_path,
     )
-    if not role:
-        return
 
     task = args.task or get_input(
         console,
         _("Agent task") + " ",
         echo_stdin=not args.no_repl,
         is_quiet=args.quiet,
+        tty_path=tty_path,
     )
     instructions = args.instructions or get_input(
         console,
         _("Agent instructions") + " ",
         echo_stdin=not args.no_repl,
         is_quiet=args.quiet,
+        tty_path=tty_path,
     )
 
     memory_recent = (
@@ -667,7 +732,7 @@ async def agent_init(args: Namespace, console: Console, theme: Theme) -> None:
         memory_recent=memory_recent,
         memory_permanent_message=memory_permanent_message,
         max_new_tokens=args.run_max_new_tokens or 1024,
-        tools=args.tool or [],
+        tools=(args.tool or []) + (getattr(args, "tools", None) or []),
     )
 
     browser_tool = get_tool_settings(
@@ -676,6 +741,12 @@ async def agent_init(args: Namespace, console: Console, theme: Theme) -> None:
         settings_cls=BrowserToolSettings,
         open_files=False,
     )
+    database_tool = get_tool_settings(
+        args,
+        prefix="database",
+        settings_cls=DatabaseToolSettings,
+        open_files=False,
+    )
 
     env = Environment(
         loader=FileSystemLoader(join(dirname(__file__), "..", "..", "agent")),
@@ -684,6 +755,8 @@ async def agent_init(args: Namespace, console: Console, theme: Theme) -> None:
     )
     template = env.get_template("blueprint.toml")
     rendered = template.render(
-        orchestrator=settings, browser_tool=browser_tool
+        orchestrator=settings,
+        browser_tool=browser_tool,
+        database_tool=database_tool,
     )
     console.print(Syntax(rendered, "toml"))
