@@ -28,6 +28,7 @@ from humanize import (
     naturalsize,
     precisedelta,
 )
+from json import dumps
 from locale import format_string
 from logging import Logger
 from math import ceil, inf
@@ -99,6 +100,7 @@ class FancyTheme(Theme):
             "tool_call_results": ":package:",
             "ttft": ":seedling:",
             "ttnt": ":alarm_clock:",
+            "ttsr": ":thinking_face:",
             "updated_at": ":calendar:",
             "user": ":hugging_face:",
             "user_input": ":speaking_head:",
@@ -1654,7 +1656,8 @@ class FancyTheme(Theme):
         tool_event_results: list[Event] | None,
         tool_running_spinner: Spinner | None,
         ttft: float,
-        ttnt: float,
+        ttnt: float | None,
+        ttsr: float | None,
         elapsed: float,
         console_width: int,
         logger: Logger,
@@ -1678,7 +1681,9 @@ class FancyTheme(Theme):
 
         pick_first = ceil(pick / 2) if pick > 1 else pick
         max_width = console_width - wrap_padding
-        think_wrapped = FancyTheme._wrap_lines(thinking_text_tokens, max_width, skip_blank_lines=True)
+        think_wrapped = FancyTheme._wrap_lines(
+            thinking_text_tokens, max_width, skip_blank_lines=True
+        )
         tool_wrapped = FancyTheme._wrap_lines(tool_text_tokens, max_width)
         wrapped = FancyTheme._wrap_lines(answer_text_tokens, max_width)
 
@@ -1758,11 +1763,19 @@ class FancyTheme(Theme):
                     (
                         _f(
                             "ttnt",
-                            _("ttnt: {ttnt} s").format(ttnt=f"{ttnt:.2f}"),
+                            _("ttnt: {ttnt} s").format(ttnt=f"{ttnt:.1f}"),
                         )
                         if ttnt
                         else None
                     ),
+                    (
+                        _f(
+                            "ttsr",
+                            _("rt: {ttsr} s").format(ttsr=f"{ttsr:.1f}"),
+                        )
+                        if ttsr
+                        else None
+                    ),
                     _f(
                         "tokens_rate",
                         _("{tokens_rate} t/s").format(
@@ -1820,9 +1833,12 @@ class FancyTheme(Theme):
                 ]
             )
         )
-        think_pannel = (
+        think_panel = (
             Panel(
-                f"[bright_black]{think_wrapped_output}[/bright_black]",
+                Align(
+                    f"[light_pink1]{think_wrapped_output}[/light_pink1]",
+                    vertical="top",
+                ),
                 title=_("{model_id} reasoning").format(
                     model_id=_f("id", model_id)
                 ),
@@ -1843,7 +1859,7 @@ class FancyTheme(Theme):
         )
         tool_panel = (
             Panel(
-                f"[cyan]{tool_wrapped_output}[/cyan]",
+                Align(f"[cyan]{tool_wrapped_output}[/cyan]", vertical="top"),
                 title=_("Tool call requests"),
                 title_align="left",
                 height=(
@@ -1862,7 +1878,7 @@ class FancyTheme(Theme):
         )
         answer_panel = (
             Panel(
-                wrapped_output,
+                Align(wrapped_output, vertical="top"),
                 title=(
                     _("{model_id} response").format(
                         model_id=_f("id", model_id)
@@ -1902,7 +1918,7 @@ class FancyTheme(Theme):
                 Group(
                     *_lf(
                         [
-                            think_pannel or None,
+                            think_panel or None,
                             tool_panel or None,
                             tool_running_panel or None,
                             answer_panel or None,
@@ -2110,7 +2126,7 @@ class FancyTheme(Theme):
                 Group(
                     *_lf(
                         [
-                            think_pannel or None,
+                            think_panel or None,
                             tool_panel or None,
                             tool_running_panel or None,
                             answer_panel or None,
@@ -2232,7 +2248,11 @@ class FancyTheme(Theme):
                                             or []
                                         ),
                                         result="[spring_green3]"
-                                        + event.payload["result"].result
+                                        + (
+                                            dumps(
+                                                event.payload["result"].result
+                                            )
+                                        )
                                         + "[/spring_green3]",
                                     )
                                     if event.type == EventType.TOOL_RESULT
@@ -2406,7 +2426,9 @@ class FancyTheme(Theme):
         )
 
     @staticmethod
-    def _wrap_lines(text_tokens: list[str], width: int, skip_blank_lines: bool = False) -> list[str]:
+    def _wrap_lines(
+        text_tokens: list[str], width: int, skip_blank_lines: bool = False
+    ) -> list[str]:
         lines: list[str] = []
         output = "".join(text_tokens)
         for line in output.splitlines():
