@@ -19,12 +19,22 @@ SimilarityFunction = Literal["cosine", "dot", "euclidean", "manhattan"]
 
 ImageTextGenerationLoaderClass = Literal["gemma3", "qwen2"]
 
-TextGenerationLoaderClass = Literal["auto", "gemma3", "mistral3"]
+TextGenerationLoaderClass = Literal["auto", "gemma3", "gpt-oss", "mistral3"]
+
+
+class GenerationCacheStrategy(StrEnum):
+    DYNAMIC = "dynamic"
+    STATIC = "static"
+    OFFLOADED_STATIC = "offloaded_static"
+    SLIDING_WINDOW = "sliding_window"
+    HYBRID = "hybrid"
+    MAMBA = "mamba"
+    QUANTIZED = "quantized"
 
 
 class Backend(StrEnum):
     TRANSFORMERS = "transformers"
-    MLXLM = "mlxlm"
+    MLXLM = "mlx"
     VLLM = "vllm"
 
 
@@ -122,6 +132,7 @@ class ToolFormat(StrEnum):
     REACT = "react"
     BRACKET = "bracket"
     OPENAI = "openai"
+    HARMONY = "harmony"
 
 
 class VisionColorModel(StrEnum):
@@ -175,7 +186,12 @@ class BetaSchedule(StrEnum):
     SQUAREDCOS_CAP_V2 = "squaredcos_cap_v2"
 
 
-@dataclass(frozen=True, kw_only=True)
+class ReasoningTag(StrEnum):
+    THINK = "think"
+    CHANNEL = "channel"
+
+
+@dataclass(frozen=True, kw_only=True, slots=True)
 class QuantizationSettings:
     load_in_4bit: bool
     bnb_4bit_quant_type: Literal["nf4"]
@@ -183,7 +199,7 @@ class QuantizationSettings:
     bnb_4bit_compute_dtype: type
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class EngineSettings:
     auto_load_model: bool = True
     auto_load_tokenizer: bool = True
@@ -208,7 +224,7 @@ class EngineSettings:
     upsampler_model_id: str | None = None
 
 
-@dataclass(kw_only=True, frozen=True)
+@dataclass(kw_only=True, frozen=True, slots=True)
 class EngineUri:
     host: str | None
     port: int | None
@@ -223,7 +239,24 @@ class EngineUri:
         return not self.vendor or self.vendor == "local"
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
+class ReasoningSettings:
+    max_new_tokens: int | None = None
+    enabled: bool = True
+    stop_on_max_new_tokens: bool = False
+    tag: ReasoningTag | None = None
+
+
+@dataclass(frozen=True, kw_only=True, slots=True)
+class ChatSettings:
+    add_generation_prompt: bool = True
+    tokenize: bool = True
+    add_special_tokens: bool = True
+    return_dict: bool = True
+    enable_thinking: bool = True
+
+
+@dataclass(frozen=True, kw_only=True, slots=True)
 class GenerationSettings:
     # Generation length ------------------------------------------------------
     # The minimum numbers of tokens to generate, ignoring the number of tokens
@@ -272,6 +305,7 @@ class GenerationSettings:
     # Whether or not the model should use the past last key/values attentions
     # (if applicable to the model) to speed up decoding
     use_cache: bool = True
+    cache_strategy: GenerationCacheStrategy | None = None
 
     # Generation output variables --------------------------------------------
     # The number of independently computed returned sequences for each element
@@ -342,20 +376,22 @@ class GenerationSettings:
     enable_gradient_calculation: bool = False
     # Use async generator (token streaming)
     use_async_generator: bool = True
-    # Optional attention mask passed directly to ``generate``
-    attention_mask: Tensor | None = None
+    # Use the attention mask discovered during input tokenization
+    use_inputs_attention_mask: bool = True
     # Parameters passed to tokenizer.apply_chat_template
-    chat_template_settings: dict[str, object] | None = field(
-        default_factory=lambda: {
-            "add_generation_prompt": True,
-            "tokenize": True,
-            "add_special_tokens": True,
-            "return_dict": True,
-        }
-    )
+    chat_settings: ChatSettings = field(default_factory=ChatSettings)
+    reasoning: ReasoningSettings = field(default_factory=ReasoningSettings)
+
+    # Templating ------------------------------------------------------------
+    # Additional variables available during prompt and message rendering
+    template_vars: dict | None = None
+
+    # Response settings ------------------------------------------------------
+    # How to format the model response
+    response_format: dict | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class GenericProxyConfig:
     scheme: str
     host: str
@@ -373,7 +409,7 @@ class GenericProxyConfig:
         return {"http": url, "https": url}
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class HubCacheFile:
     name: str
     path: str
@@ -382,7 +418,7 @@ class HubCacheFile:
     last_modified: datetime
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class HubCache:
     model_id: str
     path: str
@@ -393,7 +429,7 @@ class HubCache:
     total_revisions: int
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class HubCacheDeletion:
     model_id: str
     revisions: list[str]
@@ -404,25 +440,50 @@ class HubCacheDeletion:
     deletable_snapshots: list[str]
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class ImageEntity:
     label: str
     score: float | None = None
     box: list[float] | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
+class MessageContentText:
+    type: Literal["text"]
+    text: str
+
+
+@dataclass(frozen=True, kw_only=True, slots=True)
+class MessageContentImage:
+    type: Literal["image_url"]
+    image_url: dict[str, str]
+
+
+MessageContent = MessageContentText | MessageContentImage
+
+
+@dataclass(frozen=True, kw_only=True, slots=True)
+class MessageToolCall:
+    id: str | None = None
+    name: str
+    arguments: list
+    content_type: Literal["json"] = "json"
+
+
+@dataclass(frozen=True, kw_only=True, slots=True)
 class Message:
     role: MessageRole
-    content: str
+    thinking: str | None = ""
+    content: str | MessageContent | list[MessageContent] | None = None
     name: str | None = None
     arguments: dict | None = None
+    tool_calls: list[MessageToolCall] | None = None
 
 
 Input = str | list[str] | Message | list[Message]
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class ReasoningOrchestratorResponse:
     """Result returned by :class:`ReasoningOrchestrator`."""
 
@@ -430,7 +491,7 @@ class ReasoningOrchestratorResponse:
     reasoning: str | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class EngineMessage:
     agent_id: UUID
     model_id: str
@@ -441,12 +502,12 @@ class EngineMessage:
         return self.message.role == MessageRole.ASSISTANT
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class EngineMessageScored(EngineMessage):
     score: float
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class Model:
     id: str
     parameters: int | None
@@ -472,7 +533,7 @@ class Model:
     updated_at: datetime
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class ModelConfig:
     # Model architectures that can be used with the model pretrained weights
     architectures: list[str] | None
@@ -540,7 +601,7 @@ class ModelConfig:
     tokenizer_class: str | None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class OrchestratorSettings:
     agent_id: UUID
     orchestrator_type: str | None
@@ -562,14 +623,14 @@ class OrchestratorSettings:
     log_events: bool
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class SearchMatch:
     query: str
     match: str
     l2_distance: float
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class SentenceTransformerModelConfig:
     backend: Literal["torch", "onnx", "openvino"]
     similarity_function: SimilarityFunction | None
@@ -577,7 +638,7 @@ class SentenceTransformerModelConfig:
     transformer_model_config: ModelConfig
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class Similarity:
     cosine_distance: float
     inner_product: float
@@ -586,7 +647,7 @@ class Similarity:
     pearson: float
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class OperationAudioParameters:
     path: str
     reference_path: str | None = None
@@ -594,7 +655,7 @@ class OperationAudioParameters:
     sampling_rate: int
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class OperationTextParameters:
     context: str | None = None
     labeled_only: bool | None = None
@@ -607,7 +668,7 @@ class OperationTextParameters:
     system_prompt: str | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class OperationVisionParameters:
     path: str
     reference_path: str | None = None
@@ -639,7 +700,7 @@ class OperationParameters(TypedDict, total=False):
     vision: OperationVisionParameters | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class Operation:
     generation_settings: GenerationSettings | None
     input: Input | None
@@ -648,7 +709,7 @@ class Operation:
     requires_input: bool = False
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class TokenizerConfig:
     name_or_path: str
     tokens: list[str] | None
@@ -659,21 +720,21 @@ class TokenizerConfig:
     fast: bool = False
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class Token:
     id: Tensor | int
     token: str
     probability: float | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class TokenDetail(Token):
     step: int | None = None
     probability_distribution: ProbabilityDistribution | None = None
     tokens: list[Token] | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class ReasoningToken(Token):
     """Token produced while the model is reasoning."""
 
@@ -684,10 +745,10 @@ class ReasoningToken(Token):
         id: Tensor | int = -1,
         probability: float | None = None,
     ) -> None:
-        super().__init__(id=id, token=token, probability=probability)
+        Token.__init__(self, id=id, token=token, probability=probability)
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class ToolCallToken(Token):
     """Token produced while the model emits a tool call."""
 
@@ -698,17 +759,17 @@ class ToolCallToken(Token):
         id: Tensor | int = -1,
         probability: float | None = None,
     ) -> None:
-        super().__init__(id=id, token=token, probability=probability)
+        Token.__init__(self, id=id, token=token, probability=probability)
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class ToolCall:
     id: UUID
     name: str
     arguments: dict[str, ToolValue] | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class ToolCallContext:
     agent_id: UUID | None = None
     input: Input | None = None
@@ -717,14 +778,14 @@ class ToolCallContext:
     calls: list[ToolCall] | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class ToolCallResult(ToolCall):
     id: UUID
     call: ToolCall
     result: ToolValue | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class ToolFilter:
     func: Callable[
         [ToolCall, ToolCallContext], tuple[ToolCall, ToolCallContext] | None
@@ -732,7 +793,7 @@ class ToolFilter:
     namespace: str | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class ToolTransformer:
     func: Callable[
         [ToolCall, ToolCallContext, ToolValue | None], ToolValue | None
@@ -740,7 +801,7 @@ class ToolTransformer:
     namespace: str | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class ToolManagerSettings:
     eos_token: str | None = None
     tool_format: ToolFormat | None = None
@@ -767,33 +828,34 @@ class ToolManagerSettings:
     ) = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class TextPartition:
     data: str
     total_tokens: int
     embeddings: ndarray
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class TransformerEngineSettings(EngineSettings):
     attention: AttentionImplementation | None = None
     backend: Backend = Backend.TRANSFORMERS
     loader_class: TextGenerationLoaderClass | None = "auto"
     local_files_only: bool = False
     low_cpu_mem_usage: bool = False
+    output_hidden_states: bool = False
     special_tokens: list[str] | None = None
     state_dict: dict[str, Tensor] = None
     tokens: list[str] | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class User:
     name: str
     full_name: str | None = None
     access_token_name: str | None = None
 
 
-@dataclass(frozen=True, kw_only=True)
+@dataclass(frozen=True, kw_only=True, slots=True)
 class WebshareProxyConfig:
     host: str
     port: int
